function [H] = create_lexicoH(m, n, h0_2d, factor)
% Given 2D blur kernel, create the linear operator matrix H
% Input:
%   x_2d: 2D image
%   h0_2d: 2D (blur) kernel
% Output:
%   H: operation matrix H with lexicographic notation
%   h_2d: 2D kernel with the size of x_2d
%
% Author: Seunghwan Yoo

% disp('create_lexicoH()');
% [m,n] = size(x_2d);

m_s = m / factor;
n_s = n / factor;

kernel_size = size(h0_2d, 1);
half_kernel_size = (kernel_size - 1) / 2;
kernel_mid = half_kernel_size + 1;

kk = 1;

H = sparse(m * n, m_s * n_s);

for j = 1 : n_s
%     tic;
    for i = 1 : m_s
        
        ii = (i-1) * factor + 1 + (factor - 1) / 2;
        jj = (j-1) * factor + 1 + (factor - 1) / 2;
        
        i_start = max(ii - half_kernel_size, 1);
        i_end = min(ii + half_kernel_size, m);
        j_start = max(jj - half_kernel_size, 1);
        j_end = min(jj + half_kernel_size, n);
        h0_2d_cur = h0_2d(kernel_mid-(ii-i_start) : kernel_mid+(i_end-ii), kernel_mid-(jj-j_start) : kernel_mid+(j_end-jj));
        h0_2d_cur = h0_2d_cur / sum(sum(h0_2d_cur));
        w_current = zeros(m, n);
        w_current(i_start : i_end, j_start : j_end) = h0_2d_cur;
        
        H(:, kk) = hyperConvert2d(w_current);
        kk = kk + 1;
    end
%     toc;
end


% h_2d = create_h2d(x_2d,h0_2d);
% 
% % h_row = zeros(m,m*n);
% h_row = sparse(m,m*n);
% for i=0:m-1
%     h_2d_shift = circshift(h_2d,[i,0]);
%     h_row(i+1,:) = hyperConvert2d(h_2d_shift);
% %     h_row(i+1,:) = h_2d_shift(:)';
% end
% 
% 
% for j=0:n-1
%     h_row_shift = circshift(h_row,[0,j*m]);
%     H(j*m+1:(j+1)*m,:) = h_row_shift;
% end